--[[
**************************************************************************
*     Module d'Export de donnes pour SIOC, et le KaTZ-Pit               *
*     Par KaTZe     -         http://www.3rd-wing.net                    *
*     Version RW24.0.01  du   22/11/2014  pour le Mi8              *
**************************************************************************
--]]

-- siocConfig.lua contient :
-- Script de configuration SIOC
-- Paramtres IP : Host, Port
-- Ainsi que la plage d'Offset utilise pour les valeurs KaTZ-Pit 
-- Si l'on veut dcaler la plage rentrer une valeur (ex: 2000)

dofile ( lfs.writedir().."Scripts\\siocConfig.lua" )

-- Debug Mode, si True un fichier ".csv" est cr dans le rpertoire
-- Saved Games\DCS\Export
-- Fichier Type "KTZ-SIOC3000_ComLog-yyyymmjj-hhmm.csv"
-- Info. envoys par la fonction logCom()

------------------------------------------------------------------------
--    Fonction logCom												  --
------------------------------------------------------------------------
function logCom(message)

	-- Cration du fichier de log des communication serveur, s'il n'existe pas
	-- Format , KTZ-SIOC5000_ComLog-yyyymmdd-hhmm.csv
	--
	if DEBUG_MODE and not fichierComLog then
       	fichierComLog = io.open(lfs.writedir().."Export\\KTZ-SIOC5001_ComLog-"..os.date("%Y%m%d-%H%M")..".csv", "w");
				
		-- Ecriture de l'entte dans le fichier
		if fichierComLog then
			
			fichierComLog:write("*********************************************;\n");
			fichierComLog:write("*     Fichier Log des Communications SIOC   *;\n");
			fichierComLog:write("*     Par KaTZe  -  http://www.3rd-wing.net *;\n");
			fichierComLog:write("*     Version 5.0.01 du 23/11/2014          *;\n");
			fichierComLog:write("*********************************************;\n\n");
		end
    end
	
	-- Ecriture des donnes dans le fichier existant
	if fichierComLog then
        --fichierComLog:write(string.format(" %s ; %s",os.date("%d/%m/%y %H:%M:%S"),message),"\n");
		fichierComLog:write(string.format(" %s ; %s",os.clock(),message),"\n");
	end
end


------------------------------------------------------------------------
-- 	Login to SIOC													  --
------------------------------------------------------------------------
    
function Sioc_connect()
	
	-- Va chercher la config IP dans siocConfig
    host = siocConfig.hostIP or "127.0.0.1"
    port = siocConfig.hostPort or 8092
	
	logCom("fonction Sioc_connect")
    

		c = socket.try(socket.connect(host, port)) -- connect to the listener socket
		c:setoption("tcp-nodelay",true) -- set immediate transmission mode
		c:settimeout(.01) -- set the timeout for reading the socket)
	
	
	
   
------------------------------------------------------------------------
-- 	Offset de SIOC qui seront couts								  --
-- 	0001 = Commande gnrale										  --
-- 	0002 = Commande spciale										  --
------------------------------------------------------------------------

	inputsTable = {}
	inputsTable [1]=1
	inputsTable [2]=2
	
	local lSIOC_Var, i
    local lSIOC_SendString = ""
    
    -- Generate the init string to SIOC (to register with SIOC which parameters we are interested in)
    for lSIOC_Var,i in pairs(inputsTable)
    do
        lSIOC_SendString = lSIOC_SendString..lSIOC_Var..":"
    end
	
    -- Send the initstring
    socket.try(c:send("Arn.Inicio:"..lSIOC_SendString.."\n"))
	local messageContact = ("Arn.Inicio:"..lSIOC_SendString.."\n")
	
	messageInit = "INIT-->;" .. messageContact
	
end


   
------------------------------------------------------------------------
-- 	Send Data to SIOC												  --
------------------------------------------------------------------------
function envoyerInfo(strAttribut,valeur)


		-- Dcalage des exports vers une plage SIOC
		-- Indiquer dans siocConfig.lua la plage dsire
		newAtt = tonumber(strAttribut) + siocConfig.plageSioc
		local strNew = tostring(newAtt)
		
		local strValeur = string.format("%d",valeur);
		
		if (strValeur ~= Data_Buffer[strNew]) then
			-- On stock la nouvelle valeur dans la table buffer
			Data_Buffer[strNew] = strValeur ;
			-- Envoi de la nouvelle valeur
			socket.try(c:send(string.format("Arn.Resp:%s=%.0f:\n",strNew,strValeur)))
			local messageEnvoye = "OUT--> ;" .. (string.format("Arn.Resp:%s=%.0f:",strNew,strValeur))
			-- Log du message envoy
			-- logCom(messageEnvoye)
		end		
	end


------------------------------------------------------------------------
-- 	Get Data from SIOC												  --
------------------------------------------------------------------------	
function Reception_SIOC_Cmd()
	
	-- Check for data/string from the SIOC server on the socket
    --logCom("*** Fonction recupInfo activated","\n")
	
	-- socket.try(c:send("Arn.Resp"))
	local messageRecu = c:receive()
    
	if messageRecu then
		
		local messagelog = "IN-->;".. tostring(messageRecu)
		--logCom(messagelog)
		
		local s,l,typeMessage = string.find(messageRecu,"(Arn.%w+)");
		typeMessage = tostring(typeMessage);
        
		------------------------------------------------------------
		-- Les types de message accepts :                        --
		--                                                        --
		-- Arn.Vivo   : Le serveur  reu "Arn.Vivo": du client   -- implementation  tester
		--              Le serveur rpond "Arn.Vivo"              --
		--														  --
		-- Arn.Resp   : Message pour l'execution des commandes    -- rponse de SIOC
		--              seul deux valeurs sont acceptes:         --
		--				0=[valeur] -> le paramtres valeur(      )--
		-- 				1=[valeur] -> le paramtre commande		  --
		--				Ex:Arn.Resp:0=5000:1=3: ou Arn.Resp:1=145:--
		--              a noter que Arn.Resp:1=0: remets le       --
		--              cache valeur  'nil' aussi aprs chaque   --
		--				commande excut                          --
		------------------------------------------------------------
		if typeMessage == "Arn.Resp" then
			--logData("Message type Arn.Resp-----", "\n")
			--------------------------------------------------------
			-- Type = Arn.Resp:                                   --
			--------------------------------------------------------
			
			-- extraction du message 
			-- (message type par exemple :1=3:0=23:6=3456)
			local debut,fin,message = string.find(messageRecu,"([%d:=-]+)")
			--logCom(message)
			-- longueur du message
			local longueur
			longueur = fin - debut
			--logCom(longueur)
			-- dcoupe du message en commande et envoi  lockon
			-- (commandes type 1=3  0=23  6=3456)
			
			-- KaTZe Modif BS2, Commande cod sur 8 caracteres , TDDBBBVV
			-- DD = Device
			-- BBB = numero du bouton
			-- T = Type de bouton
			-- VV = Valeur recu
			
			local commande,Schan,chan,Svaleur,valeur,i,a,b,c,d,e,f,lim,device,bouton,typbouton,val
			lim = 0

			while lim < longueur do
				a,b,commande = string.find(message,"([%d=-]+)", lim)
				--logCom(commande)
				c,d,Schan = string.find(commande, "([%d-]+)")
				chan = tonumber(Schan)
				--logCom(string.format(" Offset = %.0f",chan,"\n"))
				e,f,Svaleur = string.find(commande, "([%d-]+)",d+1)
								
				valeur = tonumber(Svaleur)
				--logCom(string.format(" Valeur = %.0f",valeur,"\n"))
				
				if chan ==1 and valeur > 0 then
						
					-- Envoi  LockOn, commande type Classique FC3
					LoSetCommand(valeur)
				
				end
				
				if chan ==2 and valeur > 0 then
					typbouton = tonumber(string.sub(Svaleur,1,1))
					device = tonumber(string.sub(Svaleur,2,3))
					bouton = tonumber(string.sub(Svaleur,4,6))
					pas = tonumber(string.sub(Svaleur,7,7))
					val = tonumber(string.sub(Svaleur,8,8))
				
					--logCom(string.format(" Device = %.0f",device,"\n"))
					--logCom(string.format(" Bouton = %.0f",bouton,"\n"))
					--logCom(string.format(" Type = %.0f",typbouton,"\n"))
					--logCom(string.format(" Valeur = %.0f",val,"\n"))
					
					if typbouton == 1 then
						-- Type interrupteur deux voies, val = val * 1000
						-- Envoi  LockOn, commande Device, Bouton + 3000 , Argument
						GetDevice(device):performClickableAction(3000+bouton,val)
						
					end
					
					if typbouton == 2 then
						-- Type interrupteur deux voies, avec capot , val = val * 1000
						-- On ouvre, bascule, ferme
						GetDevice(device):performClickableAction(3000+bouton+1,val*1000)
						GetDevice(device):performClickableAction(3000+bouton,val*1000)
						GetDevice(device):performClickableAction(3000+bouton+1,val*1000)
						
					end
					
					if typbouton == 3 then
						-- Type interrupteur press bouton , val = val * 1000
						-- On envoie 1000 puis zero
						GetDevice(device):performClickableAction(3000+bouton,val*1000)
						GetDevice(device):performClickableAction(3000+bouton,val*0)
												
					end
					
					if typbouton == 4 then
						-- Type interrupteur rotary , 0.0 , 0.1 , 0.2 , 0.3 , ...
						-- On envoie des valeur de 0  X
						GetDevice(device):performClickableAction(3000+bouton,val/10)

												
					end
				
				end
								
				lim = b + 1
			end
			
		else
			--logData("---Log: SIOC Message Incorrect ; non type Arn.Resp ; Message Ignor -----", "\n")
		end
    end
end



------------------------------------------------------------------------
-- 	Export de LockOn												  --
------------------------------------------------------------------------
function Envoi_Data_SIOC_fast()
	    -- Export  la 200ms
		--logCom ("time de la boucle 1 - Fast")
		--logCom(CurrentTime)
		
		-- Rcupration des donnes  lire --------------------
		local lMainPanel = GetDevice(0)
		
		--Check to see that the device is valid otherwise we return an emty string 
		if type(lMainPanel) ~= "table" then
			return ""
		end
		
		
		lMainPanel:update_arguments()
		
		-- ============== Clock =========================================================================
		-- Inutile, time est rcupr avec LoGetModelTime()
		--envoyerInfo(20,lMainPanel:get_argument_value(68)*24)
		--envoyerInfo(21,lMainPanel:get_argument_value(69)*60)
		--envoyerInfo(22,lMainPanel:get_argument_value(70)*60)
		
		-- ============== Parametres de Vol ===============================================================
		envoyerInfo(20,lMainPanel:get_argument_value(24)*370) 	-- IAS max speed 350km/hr
		envoyerInfo(27,lMainPanel:get_argument_value(16)*300) 	-- Vario (-30m/s , +30 m/s)
		
		envoyerInfo(24,lMainPanel:get_argument_value(19)*10000) 	-- Alti Baro 1000m
		envoyerInfo(25,lMainPanel:get_argument_value(34)*100) 	-- Alti Radar valeur non linaire
			
		
		
		--envoyerInfo(31,lMainPanel:get_argument_value(143)*1000)	-- Pitch
		--envoyerInfo(32,lMainPanel:get_argument_value(142)*1000)	-- Bank
		--envoyerInfo(35,lMainPanel:get_argument_value(11)*1000)	-- Boussole
		
		-- ============== Parametres  ==============================================================
		

		-- ============== Parametres HSI ==================================================================
		
		
		envoyerInfo(366,lMainPanel:get_argument_value(112)*3600) -- CAP (Export 0.1 degrs)
		envoyerInfo(372,lMainPanel:get_argument_value(115)*3600) -- Waypoint (Ecart par rapport  la couronne des caps)
		envoyerInfo(373,lMainPanel:get_argument_value(118)*3600) -- Course (Ecart par rapport  la couronne des caps)
		envoyerInfo(374,lMainPanel:get_argument_value(124)*3600) -- Waypoint (Ecart par rapport  la couronne des caps)
		--envoyerInfo(324,lMainPanel:get_argument_value(124)*100)
		--envoyerInfo(318,lMainPanel:get_argument_value(118)*100)
		envoyerInfo(330,lMainPanel:get_argument_value(117)*100)
		envoyerInfo(331,lMainPanel:get_argument_value(527)*100)
		envoyerInfo(364,lMainPanel:get_argument_value(528)*100) -- Waypoint Distance
		envoyerInfo(334,lMainPanel:get_argument_value(529)*100)
		envoyerInfo(335,lMainPanel:get_argument_value(530)*100)
		
		
		
		-- ============== Parametres ILS ==================================================================
		
		-- ============== Parametres Rotor =================================================================
		
		envoyerInfo(60,lMainPanel:get_argument_value(42)*110) -- Rotor rpm : max 110
		envoyerInfo(61,lMainPanel:get_argument_value(36)*140 + 10) -- Rotor pitch : gradu de 1  15 ( * 14 +1)	
				
				
		-- ============== Parametres Moteur (Fast) ================================================================
		envoyerInfo(62,lMainPanel:get_argument_value(40)*110)		-- rpm left : max 110
		envoyerInfo(63,lMainPanel:get_argument_value(41)*110)		-- rpm right : max 110
		envoyerInfo(66,lMainPanel:get_argument_value(43)*1200)		-- temp left : max 120
		envoyerInfo(67,lMainPanel:get_argument_value(45)*1200)		-- temp right : max 120
		
		envoyerInfo(56,lMainPanel:get_argument_value(37)*50 + 50)		-- mode moteur : gradu de 5  10 ( * 5 +5)	
		envoyerInfo(57,lMainPanel:get_argument_value(38)*50 + 50)		-- mode moteur : gradu de 5  10 ( * 5 +5)
		envoyerInfo(58,lMainPanel:get_argument_value(39)*100)		-- mode moteur : gradu de 5  10 ( * 5 +5)		
		
		
		
		
		-- ============== Parametres APU ===================================================================
		envoyerInfo(100,lMainPanel:get_argument_value(402) * 900)-- Temprature APU : max 900

		
				
		-- ============== Position de l'Avion ===============================================================		
		
		
		-- ============== Donnes de Navigation ===============================================================		
		


		-- ============== Parametre TWS -- En dveloppement =======================================================		
		


		
end	

function Envoi_Data_SIOC_slow()
	     -- Export  la seconde
		--logCom ("time de la boucle 2 - Slow")
		--logCom(CurrentTime)
		
		local MainPanel = GetDevice(0)
		--Check to see that the device is valid otherwise we return an emty string 
		if type(MainPanel) ~= "table" then
			return ""
		end
		MainPanel:update_arguments()
		
		-- ============== Valeur Test ============================================================		
		envoyerInfo(13,MainPanel:get_argument_value(277)*1000)		-- K : Fuel Qty Switch		
	
		-- ============== Horloge de Mission ============================================================		
		envoyerInfo(11,LoGetModelTime())-- Heure de la mission
		
		
		-- ============== Parametres Moteur (lents) ====================================================
		
		envoyerInfo(74,MainPanel:get_argument_value(115)*80)		-- Oil Pressure left
		envoyerInfo(75,MainPanel:get_argument_value(117)*80)		-- Oil Pressure right
		envoyerInfo(76,MainPanel:get_argument_value(111)*80)		-- Oil Pressure Gear Box
		envoyerInfo(70,MainPanel:get_argument_value(116)*240-60)		-- Oil Temp left : gradu de -6  18 ( * 240 - 60)	
		envoyerInfo(71,MainPanel:get_argument_value(118)*240-60)		-- Oil Temp right : gradu de -6  18 ( * 240 - 60)	
		envoyerInfo(72,MainPanel:get_argument_value(114)*200-50)		-- Oil Temp Gear Box : gradu de -5  15 ( * 200 - 50)	

    
		-- Pilototo --------------------------------------------------------------------------------
		envoyerInfo(130,MainPanel:get_argument_value(330)*1000)		-- K : Bank
		envoyerInfo(131,MainPanel:get_argument_value(331)*1000)		-- T : Pitch
		envoyerInfo(132,MainPanel:get_argument_value(332)*1000)		-- H : HDG
		envoyerInfo(133,MainPanel:get_argument_value(333)*1000)		-- B : Alt
		envoyerInfo(134,MainPanel:get_argument_value(334)*1000)		--  : FD

		envoyerInfo(136,MainPanel:get_argument_value(175)*1000)		--  : Hover
		envoyerInfo(137,MainPanel:get_argument_value(172)*1000)		--  : Descente		

		envoyerInfo(138,MainPanel:get_argument_value(82)*1000)		--  : RAlt Danger		



		-- ============== Parametres Fuel (lents) et dmarrage ==============================================
		envoyerInfo(47,MainPanel:get_argument_value(62)* 2800)-- Fuel Internal Forward : max 80		-- OK
		--envoyerInfo(48,MainPanel:get_argument_value(138)* 800)-- Fuel Internal Aft : max 80			-- OK

		--envoyerInfo(101,MainPanel:get_argument_value(162) * 1000)-- Voyant APU Valve Open
		--envoyerInfo(102,MainPanel:get_argument_value(168) * 1000)-- Voyant APU Oil Pressure
		--envoyerInfo(103,MainPanel:get_argument_value(169) * 1000)-- Voyant APU RPM
		--envoyerInfo(104,MainPanel:get_argument_value(174) * 1000)-- Voyant APU On
		
		--envoyerInfo(106,MainPanel:get_argument_value(211) * 1000)-- Voyant Cross Feed Vanne
		envoyerInfo(107,MainPanel:get_argument_value(427) * 1000)-- Voyant Vanne Gauche
		envoyerInfo(108,MainPanel:get_argument_value(429) * 1000)-- Voyant Vanne Droit
		
		--envoyerInfo(110,MainPanel:get_argument_value(200) * 1000)-- Voyant Pump AV
		--envoyerInfo(111,MainPanel:get_argument_value(201) * 1000)-- Voyant Pump AR
		--envoyerInfo(112,MainPanel:get_argument_value(202) * 1000)-- Voyant Pump IL
		--envoyerInfo(113,MainPanel:get_argument_value(203) * 1000)-- Voyant Pump IR
		--envoyerInfo(114,MainPanel:get_argument_value(185) * 1000)-- Voyant Pump EL
		--envoyerInfo(115,MainPanel:get_argument_value(186) * 1000)-- Voyant Pump ER

		--envoyerInfo(117,MainPanel:get_argument_value(163) * 1000)-- Voyant Start Valve
		envoyerInfo(118,MainPanel:get_argument_value(204) * 1000)-- Levier CutOff Left
		envoyerInfo(119,MainPanel:get_argument_value(206) * 1000)-- Levier CutOff Right
		--envoyerInfo(120,MainPanel:get_argument_value(556) * 1000)-- Levier Frein Rotor
		
		envoyerInfo(121,MainPanel:get_argument_value(416) * 10000)-- Selecteur demarrage moteur (zero=APU, 1000=Left, 2000=Right, 3000=Up maintenance)
		envoyerInfo(122,MainPanel:get_argument_value(415) * 10000)-- Type demarrage  (=Start, 1000=Vent, 2000=Crabo)
		
		envoyerInfo(64,MainPanel:get_argument_value(290) * 1000)-- EEG Left
		envoyerInfo(65,MainPanel:get_argument_value(292) * 1000)-- EEH Right
		
		envoyerInfo(123,MainPanel:get_argument_value(191) * 1000)-- Position Collectif
		 
		-- ============== Status Elments Mcaniques ========================================================
		envoyerInfo(151,MainPanel:get_argument_value(533)*100)-- Porte Cockpit , 0 ferme , 100 ouverte	
		envoyerInfo(207,MainPanel:get_argument_value(571)*1000)-- Wheel brake
		envoyerInfo(208,MainPanel:get_argument_value(473)*1000)-- brake pressure
		envoyerInfo(201,MainPanel:get_argument_value(63)*1000)-- Train AV Up
		envoyerInfo(202,MainPanel:get_argument_value(64)*1000)-- Train AV Dwn
		envoyerInfo(203,MainPanel:get_argument_value(61)*1000)-- Train R Up
		envoyerInfo(204,MainPanel:get_argument_value(62)*1000)-- Train R Dwn
		envoyerInfo(205,MainPanel:get_argument_value(59)*1000)-- Train L Up
		envoyerInfo(206,MainPanel:get_argument_value(60)*1000)-- Train L Dwn		

		-- ============== Status Armement ==================================================================

		envoyerInfo(129,MainPanel:get_argument_value(437) * 1000)-- Voyant Autoturn
		envoyerInfo(157,MainPanel:get_argument_value(438) * 1000)-- Voyant TGT Air
		envoyerInfo(158,MainPanel:get_argument_value(440) * 1000)-- Voyant TGT Mov
		envoyerInfo(159,MainPanel:get_argument_value(439) * 1000)-- Voyant TGT For
		envoyerInfo(160,MainPanel:get_argument_value(441) * 1000)-- Voyant TGT Clear		
			
		
		
		-- =============== DataLink ======================================================
		envoyerInfo(141,MainPanel:get_argument_value(21) * 1000)-- Target1
		envoyerInfo(142,MainPanel:get_argument_value(22) * 1000)-- Target2
		envoyerInfo(143,MainPanel:get_argument_value(23) * 1000)-- Target3
		envoyerInfo(144,MainPanel:get_argument_value(50) * 1000)-- Target4
		envoyerInfo(146,MainPanel:get_argument_value(17) * 1000)-- Wing1
		envoyerInfo(147,MainPanel:get_argument_value(18) * 1000)-- Wing2
		envoyerInfo(148,MainPanel:get_argument_value(19) * 1000)-- Wing3
		envoyerInfo(149,MainPanel:get_argument_value(20) * 1000)-- Wing4
		envoyerInfo(150,MainPanel:get_argument_value(16) * 1000)-- All
		envoyerInfo(152,MainPanel:get_argument_value(15) * 1000)-- Vierge
		envoyerInfo(153,MainPanel:get_argument_value(161) * 1000)-- Clear
		envoyerInfo(154,MainPanel:get_argument_value(150) * 1000)-- Ingress
		envoyerInfo(155,MainPanel:get_argument_value(159) * 1000)-- SendMem

		
		
		-- Scan du Canon slectionn ------------------------------------------------------------------------
		
		
		
		-- Scan du Panel Armement ----------------------------------------------------------------------
		
		
		-- ============== Module de Navigation =========================================================================		
		-- Module de Navigation
		
		-- ============== Module Alarme ==================================================================================		
		
		
end
	------------------------------------------------------------------------
	--    Fonction pour le calcul des coordonnes 						  --
	--	  Merci  Mnemonic                           					  --
	------------------------------------------------------------------------
	getXYCoords = function(inLatitudeDegrees, inLongitudeDegrees) -- args: 2 numbers // Return two value in order: X, Y
        local pi = 3.141592

		local zeroX = 5000000
		local zeroZ = 6600000

		local centerX = 11465000 - zeroX --circle center
		local centerZ =  6500000 - zeroZ

		local pnSxW_X = 4468608 - zeroX -- point 40dgN : 24dgE
		local pnSxW_Z = 5730893 - zeroZ

		local pnNxW_X = 5357858 - zeroX -- point 48dgN : 24dgE
		local pnNxW_Z = 5828649 - zeroZ

		local pnSxE_X = 4468608 - zeroX -- point 40dgN : 42dgE
		local pnSxE_Z = 7269106 - zeroZ

		local pnNxE_X = 5357858 - zeroX -- point 48dgN : 42dgE
		local pnNxE_Z = 7171350 - zeroZ

		local lenNorth = math.sqrt((pnNxW_X-centerX)*(pnNxW_X-centerX) + (pnNxW_Z-centerZ)*(pnNxW_Z-centerZ))
		local lenSouth = math.sqrt((pnSxW_X-centerX)*(pnSxW_X-centerX) + (pnSxW_Z-centerZ)*(pnSxW_Z-centerZ))
		local lenN_S = lenSouth - lenNorth

		local RealAngleMaxLongitude = math.atan ((pnSxW_Z - centerZ)/(pnSxW_X - centerX)) * 180/pi
		-- borders
		local EndWest = 24
		local EndEast = 42
		local EndNorth = 48
		local EndSouth = 40
		local MiddleLongitude = (EndWest + EndEast) / 2
		local ToLengthN_S = ((EndNorth - EndSouth) / lenN_S)
		local ToAngleW_E = (MiddleLongitude - EndWest) / RealAngleMaxLongitude

		local ToDegree = 360/(2*pi)
	    -- Lo coordinates system
	    local realAng = (inLongitudeDegrees - MiddleLongitude) / ToAngleW_E / ToDegree;
	    local realLen = lenSouth - (inLatitudeDegrees - EndSouth) / ToLengthN_S;
	    local outX = centerX - realLen * math.cos (realAng);
	    local outZ = centerZ + realLen * math.sin (realAng);
	    return outX, outZ
	end	


---- *************************************************************************** Main Program ********************************************************************	
------------------------------------------------------------------------
-- 	Squenceur de tche												  --
------------------------------------------------------------------------


DEBUG_MODE = true; 	-- activation du log -------------------------------
Sioc_OK = true
Data_Buffer = {}

--- *** Connexion  SIOC *** -------------------------------------------------------
logCom("Connexion  SIOC, ouverture Socket")
package.path  = package.path..";.\\LuaSocket\\?.lua"
package.cpath = package.cpath..";.\\LuaSocket\\?.dll"
socket = require("socket")
	
	
--- *** Gestion des erreurs de connection  SIOC *** --------------------------------
	if pcall(Sioc_connect) then
		logCom("SIOC Connection OK")
		Sioc_OK = true
		
	else
		logCom("SIOC Connection problme, pas de SIOC")
		Sioc_OK = false
	end


--- logCom("LogetMissionStartTime")	
StartTime = LoGetMissionStartTime()
CurrentTime = LoGetModelTime()
SamplingPeriod_1 = 0.2 -- Interval de squence rapide en secondes (dfaut 200 millisecondes)
SamplingPeriod_2 = 1   -- Interval de squence lente en secondes (dfaut 1 seconde)

-- *** Initialisation des dclencheurs rapides et lents *** -------------------------
NextSampleTime_1 = CurrentTime + SamplingPeriod_1
NextSampleTime_2 = CurrentTime + SamplingPeriod_2

--logCom("Squenceur")

KTZ_DATA =
{
-- Fonction au dmarrage mission


	KD_Start=function(self)
		
		
		
		logCom("  ","\n")
		logCom("*** Fonction KD_Start ***","\n")
		logCom(string.format(" Mission Start Time = %.0f",StartTime,"\n"))	
		logCom(string.format(" Sampling Period 1 = %.1f secondes",NextSampleTime_1,"\n"))
		logCom(string.format(" Sampling Period 2 = %.1f secondes",NextSampleTime_2,"\n"))
		
		-- local name = LoGetPilotName()
		--logCom(name)
		logCom("  ","\n")
		
		if Sioc_OK then
			logCom("*** SIOC OK ***","\n")
			-- Envoi  SIOC de l'heure de dbut de mission
			envoyerInfo(10,LoGetMissionStartTime())
			envoyerInfo(6,LoGetPlayerPlaneId())
			-- Mise  zero du panel armement dans SIOC
			--WeaponInit()
		else
		
			logCom("*** SIOC Probleme ***","\n")
			
		end
		
		
		
		
			
		
	end,

-- Fonction avant chaque image	
	KD_BeforeNextFrame=function(self)
		-- logCom(string.format("*** Fonction KD_BeforeNextFrame @= %.2f",CurrentTime,"\n"))
		-- Option Rception des ordres de SIOC  chaque image (dfaut dans la squence lente)
		-- Reception_SIOC_Cmd()
	end,
	
-- Fonction aprs chaque image
	KD_AfterNextFrame=function(self)
		-- Rcupration du Time Code, utilis par le squenceur pour test et dclancher les squences rapides et lentes
		CurrentTime = LoGetModelTime()
	end,

-- Fonction  chaque intervalle de temps type 1
-- Squence rapide : dfaut 200 millisecondes
	KD_AtInterval_1=function(self)
				
		-- logCom(string.format("*** Fonction KD_AtInterval_1 @= %.2f",CurrentTime,"\n"))
		-- calcul de la date de fin du prochain intervalle de temps
		NextSampleTime_1 = CurrentTime + SamplingPeriod_1
	
		
		if Sioc_OK then
			-- Fonction d'envoi des donnes  SIOC (liste fast)
			Envoi_Data_SIOC_fast()
		
			-- Option Rception des ordres de SIOC squence rapide (par dfaut dans la squence lente)
			Reception_SIOC_Cmd()
		
		end
	
	end,

-- Fonction  chaque intervalle de temps type 2
-- Squence lente : dfaut 1 seconde
	KD_AtInterval_2=function(self)
				
		-- logCom(string.format("*** Fonction KD_AtInterval_2 @= %.2f",CurrentTime,"\n"))
		-- calcul de la date de fin du prochain intervalle de temps
		NextSampleTime_2 = CurrentTime + SamplingPeriod_2
	
		
		if Sioc_OK then
			-- Fonction d'envoi des donnes  SIOC (liste lente)
			Envoi_Data_SIOC_slow()
		end	
		
		-- Rception des ordres de SIOC squence lente (par dfaut)
		-- Reception_SIOC_Cmd()
				
	end,	
	
-- Fonction fin de mission
	KD_Stop=function(self)
	
	-- Par dfaut, Rien ... possibilit d'imprimer un rapport de mission avec LogCom ...  dvelopper
	-- logCom(messageInit)
	logCom("*** Fonction KD_Stop ***")
	logCom("  ","\n")
		
	-- log des rsultats
	logCom(string.format(" Flight Duration = %.0f secondes",CurrentTime,"\n"))
	logCom("  ","\n")
	logCom("  ","\n")
	logCom("Miaou  tous !!!")
		
	end,
		
}


-- Declencheur de squence (depuis export.lua)
------------------------------------------------------------------------
--    Squence : Dmarrage de mission (ExportStart)    			  --
------------------------------------------------------------------------

do
	local PrevLuaExportStart=LuaExportStart;

	LuaExportStart=function()
		KTZ_DATA:KD_Start();
		
		if PrevLuaExportStart then
			PrevLuaExportStart();
		end
	end
end

------------------------------------------------------------------------
--    Squence : avant chaque image            							  --
------------------------------------------------------------------------
-- Rien par dfaut

do
	local PrevLuaExportBeforeNextFrame=LuaExportBeforeNextFrame;

	LuaExportBeforeNextFrame=function()
		-- Non actif par dfaut 
		-- KTZ_DATA:KD_BeforeNextFrame();
						
		if PrevLuaExportBeforeNextFrame then
			PrevLuaExportBeforeNextFrame();
		end
	end
end

------------------------------------------------------------------------
--    Squence : aprs chaque image            						  --
------------------------------------------------------------------------
-- On compare le time code avec les compteurs de dclenchement

do
	local PrevLuaExportAfterNextFrame=LuaExportAfterNextFrame;

	LuaExportAfterNextFrame=function()
		KTZ_DATA:KD_AfterNextFrame();
			if CurrentTime >= NextSampleTime_1 then
				KTZ_DATA:KD_AtInterval_1();  -- Dclencheur squence rapide
			end
			if CurrentTime >= NextSampleTime_2 then
				KTZ_DATA:KD_AtInterval_2();  -- Dclencheur squence lente
			end
			
		if PrevLuaExportAfterNextFrame then
			PrevLuaExportAfterNextFrame();
		end
	end
end


------------------------------------------------------------------------
--    Squence : Fin de mission (ExportStop)         				  --
------------------------------------------------------------------------

do
	local PrevLuaExportStop=LuaExportStop;

	LuaExportStop=function()
		KTZ_DATA:KD_Stop();

		if PrevLuaExportStop then
			PrevLuaExportStop();
		end
	end
end